package org.openxma.dsl.generator.mojo;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.maven.plugin.MojoExecutionException;
import org.openxma.dsl.generator.GeneratorProperties;

/**
 * Maven plugin used to execute the openXMA Dom generator workflow
 *
 * @goal generate-dom
 * @requiresDependencyResolution compile
 * @phase generate-sources
 */
public class DomGeneratorMojo extends AbstractGeneratorMojo {

    /**
     * The location of the workflow descriptor.
     *
     * @parameter default-value="workflow/DomGenerator.mwe"
     * @required
     */
    private String workflowFile;

    /**
     * The location of the custom check file.
     *
     * @parameter
     */
    private String checkFile;

    /**
     * The location of the custom template advice file.
     *
     * @parameter
     */
    private List<String> templateAdviceFiles;

    /**
     * The location of the custom extension advice file.
     *
     * @parameter
     */
    private List<String> extensionAdviceFiles;
    /**
     * @parameter default-value="${basedir}/src/main/model"
     */
    private String modelFolder;

    /**
     * @parameter default-value="${project.build.sourceDirectory}"
     */
    private String sourceOutlet;
    /**
     * @parameter default-value="${project.build.testSourceDirectory}"
     */
    private String testSourceOutlet;

    /**
     * @parameter default-value="${basedir}/src/main/resources"
     */
    private String resourcesOutlet;

    /**
     * @parameter default-value="${basedir}/src/test/resources"
     */
    private String testResourcesOutlet;

    /**
     * @parameter default-value="${basedir}/src/generated/java"
     */
    private String generatedSourceOutlet;

    /**
     * @parameter default-value="${basedir}/src/test/generated/java"
     */
    private String generatedTestSourceOutlet;

    /**
     * @parameter default-value="${basedir}/src/generated/resources"
     */
    private String generatedResourcesOutlet;

    /**
     * @parameter default-value="${basedir}/src/test/generated/resources"
     */
    private String generatedTestResourcesOutlet;

    public void doExecute() throws MojoExecutionException {
        Map<String, String> workflowParameters = getWorkflowParameterMap();
        workflowParameters.put("modelFolder", modelFolder);
        runWorkFlow(this.workflowFile, workflowParameters);
    }

    private Map<String, String> getWorkflowParameterMap() {
        Map<String, String> workflowParameters = new HashMap<String, String>();
        workflowParameters.put("cleanDirectory", Boolean.TRUE.toString());
        workflowParameters.put("doCustomChecks", Boolean.valueOf(this.checkFile != null).toString());
        workflowParameters.put("checkFile", null == this.checkFile ? "" : this.checkFile);
        workflowParameters.put("templateAdviceFiles", templateAdviceFiles != null ? join(templateAdviceFiles.toArray(),
                ";", 0, templateAdviceFiles.size()) : "");
        workflowParameters.put("extensionAdviceFiles", extensionAdviceFiles != null ? join(extensionAdviceFiles
                .toArray(), ";", 0, extensionAdviceFiles.size()) : "");
        workflowParameters.put("outlet.java", sourceOutlet);
        workflowParameters.put("outlet.test", testSourceOutlet);
        workflowParameters.put("outlet.resources", resourcesOutlet);
        workflowParameters.put("outlet.test.resources", testResourcesOutlet);
        workflowParameters.put("outlet.generated.java", generatedSourceOutlet);
        workflowParameters.put("outlet.generated.test", generatedTestSourceOutlet);
        workflowParameters.put("outlet.generated.resources", generatedResourcesOutlet);
        workflowParameters.put("outlet.generated.test.resources", generatedTestResourcesOutlet);

        return workflowParameters;
    }

    public void putWorkflowParametersToGeneratorProperties(Properties props, Map<String, String> workflowParameters) {
        // make the resource folder locations available for the hbm file post
        // processor
        if (null != workflowParameters.get(GeneratorProperties.OUTLET_RESOURCES))
            props.put(GeneratorProperties.OUTLET_RESOURCES, workflowParameters
                    .get(GeneratorProperties.OUTLET_RESOURCES));

        if (null != workflowParameters.get(GeneratorProperties.OUTLET_GENERATED_RESOURCES))
            props.put(GeneratorProperties.OUTLET_GENERATED_RESOURCES, workflowParameters
                    .get(GeneratorProperties.OUTLET_GENERATED_RESOURCES));
    }

}
